@@ -25,15 +25,15 @@ import os
 from paleomix.node import CommandNode, NodeError
 from paleomix.atomiccmd.command import AtomicCmd
 from paleomix.atomiccmd.builder import \
-     AtomicCmdBuilder, \
-     use_customizable_cli_parameters, \
-     create_customizable_cli_parameters
+    AtomicCmdBuilder, \
+    use_customizable_cli_parameters, \
+    create_customizable_cli_parameters
 
 from paleomix.atomiccmd.sets import ParallelCmds
 from paleomix.nodes.samtools import SAMTOOLS_VERSION
 from paleomix.common.fileutils import \
-     describe_paired_files, \
-     missing_files
+    describe_paired_files, \
+    missing_files
 
 import paleomix.common.versions as versions
 import paleomix.tools.factory as factory
@@ -107,10 +107,9 @@ class BWABacktrack(CommandNode):
                                for key in parameters.order])
 
         description \
-            = _get_node_description(name="BWA Backtrack",
-                                    algorithm="SE",
-                                    input_files_1=(parameters.input_file,),
-                                    input_files_2=(),
+            = _get_node_description(name="BWA",
+                                    algorithm='Backtrack',
+                                    input_files_1=parameters.input_file,
                                     prefix=parameters.prefix,
                                     threads=parameters.threads)
 
@@ -155,9 +154,7 @@ class BWASamse(CommandNode):
 
         input_file = parameters.input_file_fq
         description = _get_node_description(name="BWA Samse",
-                                            algorithm="SE",
-                                            input_files_1=(input_file,),
-                                            input_files_2=(),
+                                            input_files_1=input_file,
                                             prefix=parameters.prefix)
 
         CommandNode.__init__(self,
@@ -212,10 +209,9 @@ class BWASampe(CommandNode):
 
         input_file_1 = parameters.input_file_fq_1
         input_file_2 = parameters.input_file_fq_2
-        description = _get_node_description(name="BWA Samse",
-                                            algorithm="SE",
-                                            input_files_1=(input_file_1,),
-                                            input_files_2=(input_file_2,),
+        description = _get_node_description(name="BWA Sampe",
+                                            input_files_1=input_file_1,
+                                            input_files_2=input_file_2,
                                             prefix=parameters.prefix)
 
         CommandNode.__init__(self,
@@ -273,17 +269,17 @@ class BWAAlgorithmNode(CommandNode):
         _check_bwa_prefix(parameters.prefix)
         algorithm = parameters.algorithm.upper()
         algorithm += "_PE" if parameters.input_file_2 else "_SE"
-        description = _get_node_description(name="BWA",
-                                            algorithm=algorithm,
-                                            input_files_1=parameters.input_file_1,
-                                            input_files_2=parameters.input_file_2,
-                                            prefix=parameters.prefix)
+        desc = _get_node_description(name="BWA",
+                                     algorithm=algorithm,
+                                     input_files_1=parameters.input_file_1,
+                                     input_files_2=parameters.input_file_2,
+                                     prefix=parameters.prefix)
 
         command = ParallelCmds([cmd.finalize()
                                 for cmd in parameters.commands.itervalues()])
         CommandNode.__init__(self,
                              command=command,
-                             description=description,
+                             description=desc,
                              threads=parameters.threads,
                              dependencies=parameters.dependencies)
 
@@ -294,7 +290,8 @@ class BWAAlgorithmNode(CommandNode):
 
 def _process_output(stdin, output_file, reference, run_fixmate=False):
     convert = factory.new("cleanup")
-    convert.set_option("--fasta", "%(IN_FASTA_REF)s")
+    if reference is not None:
+        convert.set_option("--fasta", "%(IN_FASTA_REF)s")
     convert.set_option("--temp-prefix", "%(TEMP_OUT_PREFIX)s")
     convert.set_kwargs(IN_STDIN=stdin,
                        IN_FASTA_REF=reference,
@@ -340,7 +337,7 @@ def _get_max_threads(reference, threads):
     otherwise the requested number of threads is returned.
     """
     if reference not in _PREFIX_SIZE_CACHE:
-        if not os.path.exists(reference):
+        if reference is None or not os.path.exists(reference):
             _PREFIX_SIZE_CACHE[reference] = None
         else:
             _PREFIX_SIZE_CACHE[reference] = os.path.getsize(reference)
@@ -412,12 +409,22 @@ def _build_cat_command(input_file, output_file):
     return cat
 
 
-def _get_node_description(name, algorithm, input_files_1, input_files_2 = (), prefix = None, threads = 1):
-    threads_str     = ("%i threads" % (threads,)) if (threads > 1) else None
-    prefix_str      = os.path.basename(prefix) if prefix else None
-    if prefix_str.endswith(".fasta") or prefix_str.endswith(".fa"):
-        prefix_str  = prefix_str.rsplit(".", 1)[0]
-    file_desc       = describe_paired_files(input_files_1, input_files_2 or ())
-    info            = ", ".join(filter(None, (prefix_str, algorithm, threads_str)))
+def _get_node_description(name, input_files_1, input_files_2=None,
+                          algorithm=None, prefix=None, threads=1):
+    info = []
+    if prefix is not None:
+        prefix = os.path.basename(prefix)
+        if prefix.endswith(".fasta") or prefix.endswith(".fa"):
+            prefix = prefix.rsplit(".", 1)[0]
 
-    return "<%s (%s): %s>" % (name, info, file_desc)
+        info.append(prefix)
+
+    if algorithm is not None:
+        info.append(algorithm)
+
+    if threads > 1:
+        info.append("%i threads" % (threads,))
+
+    file_desc = describe_paired_files(input_files_1, input_files_2 or ())
+
+    return "<%s (%s): %s>" % (name, ", ".join(info), file_desc)
