@@ -22,19 +22,26 @@
 #
 from __future__ import print_function
 
-import os
 import errno
-import Queue
-import pickle
-import signal
 import logging
 import multiprocessing
+import os
+import pickle
+import Queue
+import signal
+import traceback
 
 import paleomix.ui
 import paleomix.logger
 
-from paleomix.node import Node
-from paleomix.nodegraph import NodeGraph, NodeGraphError
+from paleomix.node import \
+    Node, \
+    NodeError, \
+    NodeUnhandledException
+from paleomix.nodegraph import \
+    FileStatusCache, \
+    NodeGraph, \
+    NodeGraphError
 from paleomix.common.utilities import \
     safe_coerce_to_tuple, \
     fast_pickle_test
@@ -103,6 +110,7 @@ class Pypeline(object):
 
         is_ok = True
         progress_printer = paleomix.ui.get_ui(progress_ui)
+        progress_printer.max_threads = max_threads
         nodegraph.add_state_observer(progress_printer)
 
         with paleomix.ui.CommandLine() as cli:
@@ -118,7 +126,10 @@ class Pypeline(object):
                 if running:
                     progress_printer.flush()
 
-                max_threads = cli.process_key_presses(nodegraph, max_threads)
+                max_threads = cli.process_key_presses(nodegraph,
+                                                      max_threads,
+                                                      progress_printer)
+                progress_printer.max_threads = max_threads
                 _update_nprocesses(self._pool, max_threads)
 
         self._pool.close()
@@ -191,10 +202,15 @@ class Pypeline(object):
             except Exception, errors:
                 nodegraph.set_node_state(node, nodegraph.ERROR)
 
-                errors = "\n".join(("\t" + line)
-                                   for line in str(errors).strip().split("\n"))
-                self._logger.error("%s: Error occurred running command:\n%s\n",
-                                   node, errors)
+                message = [str(node),
+                           "  Error (%r) occurred running command:"
+                           % (type(errors).__name__)]
+
+                for line in str(errors).strip().split("\n"):
+                    message.append("    %s" % (line,))
+                message.append("")
+
+                self._logger.error("\n".join(message))
 
             if not errors:
                 nodegraph.set_node_state(node, nodegraph.DONE)
@@ -244,13 +260,20 @@ class Pypeline(object):
         return input_files - output_files
 
     def list_output_files(self):
+        cache = FileStatusCache()
+        nodegraph = NodeGraph(self._nodes, lambda: cache)
         output_files = {}
-        nodegraph = NodeGraph(self._nodes)
 
         def collect_output_files(node):
-            state = nodegraph.get_node_state(node)
+            state = None
+            if nodegraph.is_done(node, cache):
+                state = nodegraph.DONE
+                if nodegraph.is_outdated(node, cache):
+                    state = nodegraph.OUTDATED
+
             for filename in node.output_files:
                 output_files[os.path.abspath(filename)] = state
+
             return True
 
         self.walk_nodes(collect_output_files)
@@ -330,7 +353,7 @@ class Pypeline(object):
         if not self._interrupted:
             self._interrupted = True
             self._logger.error("\nKeyboard interrupt detected, waiting for "
-                               "current tasks to complete ... Press CTRL-C "
+                               "running tasks to complete ... Press CTRL-C "
                                "again to force termination.\n")
         else:
             self._pool.terminate()
@@ -445,6 +468,13 @@ def _call_run(key, node, config):
     bound functions (e.g. self.run)"""
     try:
         return node.run(config)
+    except NodeError:
+        raise
+    except Exception:
+        message = "Unhandled error running Node:\n\n%s" \
+            % (traceback.format_exc(),)
+
+        raise NodeUnhandledException(message)
     finally:
         # See comment in _init_worker
         _call_run.queue.put(key)
