@@ -9,8 +9,8 @@
 # copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
 #
-# The above copyright notice and this permission notice shall be included in all
-# copies or substantial portions of the Software.
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
 #
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -675,6 +675,12 @@ def test_apply_options__single_option__boolean__pop_when_value_is_false():
     apply_options(mock, {"-v": False})
 
 
+def test_apply_options__single_option__boolean__pop_missing_throws():
+    mock = flexmock()
+    mock.should_receive('pop_option').with_args('-v').and_raise(KeyError('-v'))
+    assert_raises(KeyError, apply_options, mock, {"-v": False})
+
+
 def test_apply_options__multiple_option():
     mock = flexmock()
     mock.should_receive('add_option').with_args('--foo', 3).once()
